@@ -1,18 +1,21 @@
 """
-core.py
+Core API functions and initialization routines.
+"""
 
-Created by Kang Zhang on 2009-07-09
-"""
 import os
 import sys
 import logging
 import operator
 
-from .py27compat import configparser
+from .py27compat import configparser, filter
+from .py33compat import max
 
 from . import logger
 from . import backend
 from .util import platform_ as platform
+from .util import once
+from .backends import fail
+
 
 log = logging.getLogger(__name__)
 
@@ -51,38 +54,21 @@ def delete_password(service_name, username):
     _keyring_backend.delete_password(service_name, username)
 
 
-def init_backend():
+recommended = lambda backend: backend.priority >= 1
+by_priority = operator.attrgetter('priority')
+
+
+def init_backend(limit=None):
     """
     Load a keyring specified in the config file or infer the best available.
     """
     _load_library_extensions()
-    set_keyring(load_config() or _get_best_keyring())
+    keyrings = filter(limit, backend.get_all_keyring())
 
-
-by_priority = operator.attrgetter('priority')
-
-
-def _get_recommended_keyring():
-    """
-    Get the highest priority recommended keyring or None if
-    no recommended keyring backend is viable.
-    """
-    keyrings = (
-        keyring
-        for keyring in backend.get_all_keyring()
-        if keyring.priority >= 1
+    set_keyring(
+        load_config()
+        or max(keyrings, default=fail.Keyring, key=by_priority)
     )
-    return max(keyrings, None, key=by_priority)
-
-
-def _get_best_keyring():
-    """
-    Return the best keyring backend for the given environment based on
-    priority.
-    """
-    keyrings = backend.get_all_keyring()
-    # get the most recommended one
-    return max(keyrings, key=by_priority)
 
 
 def _load_keyring_class(keyring_name):
@@ -96,7 +82,6 @@ def _load_keyring_class(keyring_name):
     ...      'keyring.backends.OS_X.Keyring',
     ...      'keyring.backends.kwallet.DBusKeyring',
     ...      'keyring.backends.SecretService.Keyring',
-    ...      'keyring.backends.Gnome.Keyring',
     ...  ]
     >>> list(map(_load_keyring_class, popular_names))
     [...]
@@ -104,7 +89,6 @@ def _load_keyring_class(keyring_name):
     These legacy names are retained for compatibility.
 
     >>> legacy_names = [
-    ...      'keyring.backends.kwallet.Keyring',
     ...  ]
     >>> list(map(_load_keyring_class, legacy_names))
     [...]
@@ -162,6 +146,7 @@ def _load_keyring_path(config):
     except (configparser.NoOptionError, configparser.NoSectionError):
         pass
 
+@once
 def _load_library_extensions():
     """
     Locate all setuptools entry points by the name 'keyring backends'
